from typing import Optional

import anthropic
from anthropic.types import (
    Message,
    MessageParam,
    TextBlock,
    ToolResultBlockParam,
    ToolUseBlock,
)
from typing_extensions import assert_never

from fi.integrations.otel import AnthropicInstrumentor, register
from fi.integrations.otel.types import ProjectType
from fi.test.eval_tags import eval_tags

# Configure trace provider with custom evaluation tags
trace_provider = register(
    eval_tags=eval_tags,
    project_name="ANTHROPIC_TEST 1",
    project_version_name="project version v2",
)

# Initialize the Anthropic instrumentation
AnthropicInstrumentor().instrument(tracer_provider=trace_provider)


def _to_assistant_message_param(
    message: Message,
) -> MessageParam:
    content = []
    for block in message.content:
        if isinstance(block, TextBlock):
            content.append(block)
        elif isinstance(block, ToolUseBlock):
            content.append(block)
        else:
            assert_never(block)
    return MessageParam(content=content, role="assistant")


def _get_tool_use_id(message: Message) -> Optional[str]:
    for block in message.content:
        if isinstance(block, ToolUseBlock):
            return block.id
    return None


# Initialize Anthropic client
client = anthropic.Anthropic()
messages = [
    {
        "role": "user",
        "content": "What is the weather like in San Francisco in Fahrenheit?",
    }
]
response = client.messages.create(
    model="claude-3-5-sonnet-20240620",
    max_tokens=1024,
    tools=[
        {
            "name": "get_weather",
            "description": "Get the current weather in a given location",
            "input_schema": {
                "type": "object",
                "properties": {
                    "location": {
                        "type": "string",
                        "description": "The city and state, e.g. San Francisco, CA",
                    },
                    "unit": {
                        "type": "string",
                        "enum": ["celsius", "fahrenheit"],
                        "description": 'The unit of temperature, either "celsius" or "fahrenheit"',
                    },
                },
                "required": ["location"],
            },
        }
    ],
    messages=messages,
)
messages.append(_to_assistant_message_param(response))

assert (tool_use_id := _get_tool_use_id(response)) is not None, "tool was not called"
messages.append(
    MessageParam(
        content=[
            ToolResultBlockParam(
                tool_use_id=tool_use_id,
                content='{"weather": "sunny", "temperature": "75"}',
                type="tool_result",
                is_error=False,
            )
        ],
        role="user",
    )
)
response = client.messages.create(
    model="claude-3-5-sonnet-20240620",
    max_tokens=1024,
    tools=[
        {
            "name": "get_weather",
            "description": "Get the current weather in a given location",
            "input_schema": {
                "type": "object",
                "properties": {
                    "location": {
                        "type": "string",
                        "description": "The city and state, e.g. San Francisco, CA",
                    },
                    "unit": {
                        "type": "string",
                        "enum": ["celsius", "fahrenheit"],
                        "description": 'The unit of temperature, either "celsius" or "fahrenheit"',
                    },
                },
                "required": ["location"],
            },
        }
    ],
    messages=messages,
)
# print(f"{response=}")
